/*
 * makeaddtlist.c
 *
 * Yes it was a typo, but who cares.
 *
 * This is a modification of the CIDR calc code from
 * FreeBSD.  Cut down and modified for this app.
 *
 */

/* Debug Header */
#if DEBUG
# define IFDEBUG(x) do { if (debug) x; }while(0)
# define IFDEBUG5(x) do { if (debug) x; }while(0)
#else
# define IFDEBUG(x)
# define IFDEBUG5(x)
#endif
/* End Debug Header */

#include <stdio.h>
#include <stdlib.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

void usage(char *, int);
void write_addrs(char *);
void range_error(int);
void invalid(int);
void hosts(unsigned long int, unsigned long int,
		unsigned long int, int, struct in_addr,
		struct in_addr, FILE * );

int singlehost=0;

int main(int argc, char **argv) {

	char filename[64] = "";
	FILE *out;

	struct in_addr in;
	struct in_addr ma;
	struct in_addr lowhost;
	struct in_addr highhost;

	unsigned long int address;
	unsigned long int mask;
	unsigned long int maskval;
	unsigned long int i,temp;

	int binaddr=1;
	int decaddr=1;
	int hexaddr=1;
	int quadaddr=1;
	int binmask=1;
	int decmask=1;
	int hexmask=1;
	int quadmask=1;
	int prefix=1;
	int validhex=1;
	int hostlist=1;
	int cidraddress=1;
	int cidrprefix=1;
	int cidrhostlist=1;
	int getversion=1;

	double range;

	char *ip[3];
	char *holdaddress;
	char *holdprefix;
	const char *sep="/";

	if (argc < 2) usage(argv[0], 1);

	for(i=0;i<3;i++)
		ip[i]=(char *)NULL;

	holdaddress=(char *)NULL;
	holdprefix=(char *)NULL;
	
	if(strlen(argv[1])>18) {
		fprintf(stderr,"argv[1] too long\n");
		exit(1);
	}

	for(i=0;i<3;i++) {
		if( (ip[i]=malloc(strlen(argv[1])+1)) == NULL ) {
			fprintf(stderr,"ip[i] malloc() failed\n");
			exit(1);
		}
	}

	if( (holdaddress=malloc(strlen(argv[1])+1)) == NULL ) {
		fprintf(stderr,"holdaddress malloc() failed.\n");
		exit(1);
	}

	if( (holdprefix=malloc(strlen(argv[1])+1)) == NULL ) {
		fprintf(stderr,"holdprefix malloc() failed.\n");
		exit(1);
	}

	strncpy(ip[2],argv[1],18);

	holdaddress=strtok(ip[2],sep);
	if(holdaddress==NULL) invalid(1);

	strncpy(ip[0],holdaddress,strlen(holdaddress));
	if(ip[0]==NULL) invalid(1);

	holdprefix=strtok(NULL,sep);
	if(holdprefix==NULL) invalid(1);

	strncpy(ip[1],holdprefix,strlen(holdprefix));
	if(ip[1]==NULL) invalid(1);

	maskval=0xffffffff;
	maskval>>=(32-atoi(ip[1]));
	maskval<<=(32-atoi(ip[1]));
	
	if( inet_aton(ip[0],&in) == 0 ) invalid(1);

	if(atoi(ip[1])<0||atoi(ip[1])>32) range_error(1);

	if(atoi(ip[1])==32) singlehost=1;

	mask=maskval;

	sprintf(filename, "%s-%s.results", inet_ntoa(in), ip[1]); 
	if((out = fopen(filename, "w+")) == NULL) {
		fprintf(stderr, "Error: can't write to outfile, %s\n", argv[2]);
		exit(-1);
	}

	printf("\nip address..........:  %s\n",inet_ntoa(in));
	in.s_addr&=htonl(mask);
	ma.s_addr=htonl(mask);
	printf("netmask.............:  %s\n\n",inet_ntoa(ma));
	printf("network address.....:  %s\n",inet_ntoa(in));

	lowhost=in;

	in.s_addr|=~htonl(mask);

	highhost=in;
	printf("broadcast address...:  %s\n\n",inet_ntoa(in));
	hosts(i,temp,mask,cidrhostlist,lowhost,highhost,out);
	for(i=0;i<3;i++) {
		if(ip[i]!=NULL) {
			ip[i]=(char *)NULL;
			free(ip[i]);
		}
	}

	if(holdaddress!=NULL) {
		holdaddress=(char *)NULL;
		free(holdaddress);
	}

	if(holdprefix!=NULL) {
		holdprefix=(char *)NULL;
		 free(holdprefix);
	}

	return 0;

}

void range_error(int argnum) {

	printf("range error: argv[%d]\n",argnum);
	exit(-1);

}

void invalid(int argnum) {

	printf("invalid argument: argv[%d]\n",argnum);
	exit(-1);

}

void hosts(unsigned long int i_val, unsigned long int temp_val,
		unsigned long int mask_val, int hostlist_val,
		struct in_addr lowhost_val, struct in_addr highhost_val, FILE *out) {
	
	unsigned long int j=0;

	printf("Please wait while testing host addresses...\n\n");
	
	temp_val=ntohl(lowhost_val.s_addr);
	for(hostlist_val=0; hostlist_val<2; hostlist_val++) {

		if(hostlist_val) printf("Total host addresses:  %lu\n\n",j);

		if (singlehost) {
			if (hostlist_val)
				scan(inet_ntoa(lowhost_val), j, out);
			else
				++j;
		}


		for(i_val=temp_val+1;i_val<ntohl(highhost_val.s_addr);++i_val) {
			lowhost_val.s_addr=htonl(i_val);
			if(!(htonl(temp_val)^(lowhost_val.s_addr&htonl(mask_val)))) {
				if(hostlist_val)
					scan(inet_ntoa(lowhost_val), j, out);
				else
					++j;
			}
		}
	}
	printf("                                                                            \nDone.\n");
}

void usage(char *name, int ev) {

	printf("\nUsage: %s <CIDR Formatted Address>\n\n", name);
	printf("Examples:\n");
	printf("\t%s 192.168.0.1/24\n", name);
	printf("\t%s 172.16.0.0/16\n", name);
	printf("\t%s 10.12.0.1/14\n", name);
	printf("\nSpecial case is: 192.168.0.1/32 which will now scan a single host\n\n");
	exit(ev);

}
